function z = membrana_multigrid(bordo, carico, r)
	% z = membrana_multigrid(bordo, carico, r)
	% Esegue r passi del metodo di Jacobi per il problema della membrana elastica, senza costruire esplicitamente la matrice A mn x mn
	% La soluzione iniziale per il metodo iterativo è data della soluzione approssimata con r passi di Jacobi su una griglia con maglie più larghe; la funzione assume m, n dispari
	% bordo: matrice m x n con i valori sul bordo uguali alle altezze della membrana sul bordo
	% carico: matrice m x n tale che carico(i,j) è il peso posto nel punto (i,j)
	% z: matrice m x n con i valori approssimati delle altezze della membrana calcolati dopo r passi del metodo di Jacobi

	[m, n] = size(bordo);

	% Inizializzo bordo e carico per problema più piccolo:
	bordo_coarse = bordo(1:2:end, 1:2:end);
	carico_coarse = carico(1:2:end, 1:2:end);
	% Divido il carico dei punti non presenti nella griglia più piccola tra i punti adiacenti:
	carico_coarse(2:end-1, 2:end-1) = carico_coarse(2:end-1, 2:end-1) + ...
				1/2 * (carico(2:2:end-3, 3:2:end-2) + carico(4:2:end-1, 3:2:end-2) + ...
				carico(3:2:end-2, 2:2:end-3) + carico(3:2:end-2, 4:2:end-1)) + ...
				1/4 * (carico(2:2:end-3, 2:2:end-3) + carico(4:2:end-1, 2:2:end-3) + ...
				carico(4:2:end-1, 2:2:end-3) + carico(4:2:end-1, 4:2:end-1));

	% Risolvo problema più piccolo con r passi di Jacobi:
	z_coarse = membrana(bordo_coarse, carico_coarse, r);
	
	% Inizializzo z usando z_coarse e interpolazione lineare per i punti intermedi:
	z = bordo;
	z(1:2:end, 1:2:end) = z_coarse;
	z(2:2:end-1, 3:2:end-2) = 1/2 * (z(1:2:end-2, 3:2:end-2) + z(3:2:end, 3:2:end-2));
	z(3:2:end-2, 2:2:end-1) = 1/2 * (z(3:2:end-2, 1:2:end-2) + z(3:2:end-2, 3:2:end));
	z(2:2:end-1, 2:2:end-1) = 1/4 * (z(1:2:end-2, 2:2:end-1) + z(3:2:end, 2:2:end-1) + ...
								z(2:2:end-1, 1:2:end-2) + z(2:2:end-1, 3:2:end));
	
	% Risolvo problema più grande partendo da z (r iterazioni di Jacobi):
	for k = 1:r
		z(2:end-1, 2:end-1) = -1/4 * (carico(2:end-1, 2:end-1) - z(1:end-2, 2:end-1) - z(3:end, 2:end-1) - z(2:end-1, 1:end-2) - z(2:end-1, 3:end));
	end
	
end
